﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class PlayerShip : MonoBehaviour {

    public float amplitude;
    public float frequency;
    private float _frequency;
    private float phase = 0.0f;
    float time = 0f;
    public float speed;

    public float timer = 0;

    float baseX;
    float baseY;
    float offsetX;
    public float horMoveSpeed;
    public float frequencyShiftSpeed;
    public float offsetXLeftLimit;
    public float offsetXRightLimit;
    public float minFrequency;
    public float maxFrequency;

    public Projectile laser;
    public GameObject shield;
    public GameObject shipNormal;
    public GameObject shipDamaged;
    public float shieldMin;
    public float normalMin;

    public float health;
    public float reloadTimer;

    public float damageDelay;
    public float damageTimer;

    public int lives;
    public float deathDelay;
    float deathTimer;
    bool deathPeriod;

    public float deathFlashPeriod;
    public float deathFlashTimer;

    AudioSource audioSource;

    // Use this for initialization
    void Start () {
        _frequency = frequency;
        baseX = transform.position.x;
        baseY = transform.position.y;
        audioSource = GetComponent<AudioSource>();
    }
	
	// Update is called once per frame
	void Update () {
        timer += Time.deltaTime;
        damageTimer -= Time.deltaTime;
        deathTimer -= Time.deltaTime;
        reloadTimer -= Time.deltaTime;
        deathFlashTimer -= Time.deltaTime;

        if (deathTimer <=0  && deathPeriod)
        {
            health = 3;
            shipDamaged.SetActive(true);
            shipNormal.SetActive(true);
            shield.SetActive(true);
            deathPeriod = false;
        }

        if (deathPeriod && deathTimer > 0)
        {
            if (deathFlashTimer < 0)
            {
                if (shipNormal.activeSelf)
                {
                    shipNormal.SetActive(false);
                    shield.SetActive(false);
                } else
                {
                    shipNormal.SetActive(true);
                    shield.SetActive(true);
                }
                deathFlashTimer = deathFlashPeriod;
            }
        }

        if (Input.GetKey(KeyCode.LeftArrow) && offsetX > offsetXLeftLimit)
        {
            offsetX -= horMoveSpeed;
        }
        if (offsetX < offsetXLeftLimit)
        {
            offsetX = offsetXLeftLimit;
        }
        if (Input.GetKey(KeyCode.RightArrow) && offsetX < offsetXRightLimit)
        {
            offsetX += horMoveSpeed;
        }
        if (offsetX > offsetXRightLimit)
        {
            offsetX = offsetXRightLimit;
        }

        if (Input.GetKey(KeyCode.DownArrow) && frequency > minFrequency)
        {
            frequency -= frequencyShiftSpeed;
        }
        if (frequency < minFrequency)
        {
            frequency = minFrequency;
        }
        if (Input.GetKey(KeyCode.UpArrow) && frequency < maxFrequency)
        {
            frequency += frequencyShiftSpeed;
        }
        if (frequency > maxFrequency)
        {
            frequency = maxFrequency;
        }

        if (!deathPeriod)
        {
            

            if (Input.GetKey(KeyCode.Z) && reloadTimer <= 0)
            {
                Projectile shoot = Instantiate(laser);
                shoot.transform.position = transform.position;
                shoot.setInitialPos(transform.position.x, transform.position.y);
                shoot.damage = frequency;
                audioSource.Play();
                reloadTimer = frequency / 3;
            }
        }
        
        if (frequency != _frequency)
            CalcNewFreq();

        float yPos = baseY + Mathf.Sin(timer * _frequency + phase) * amplitude;
        float xPos = baseX + (timer * speed) + offsetX;

        transform.position = new Vector2(xPos,yPos);
    }


    void CalcNewFreq()
    {
        float curr = (timer * _frequency + phase) % (2.0f * Mathf.PI);
        float next = (timer * frequency) % (2.0f * Mathf.PI);
        phase = curr - next;
        _frequency = frequency;
    }

    //phase getter for sin visualization.
    public float GetPhase() {
        return phase;
    }

    public float GetFrequency() {
        return _frequency;
    }

    public void TakeDamage(float damage)
    {
        if (damageTimer <= 0 && !deathPeriod)
        {
            health -= damage;
            damageTimer = damageDelay;
            if (damage < shieldMin)
            {
                shield.SetActive(false);
            } else
            {
                shield.SetActive(true);
            }

            if (damage < normalMin)
            {
                shipNormal.SetActive(false);
            }
            else
            {
                shipNormal.SetActive(true);
            }

            if (health <= 0)
            {
                LifeLost();
            }
        }
    }

    void LifeLost()
    {
        deathTimer = deathDelay;
        deathTimer -= Time.deltaTime;
        deathPeriod = true;

        shipDamaged.SetActive(false);
        offsetX = 0;

        if (lives < 0)
        {

        } else
        {
            lives--;
        }
    }
}
