﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class WaveDisplay : MonoBehaviour {

    public int resolution = 10;
    public int length;

    private int currentResolution;
    private ParticleSystem particles;
    private ParticleSystem.Particle[] points;

    public float amplitude;
    public float frequency;
    private float _frequency;
    private float phase = 0.0f;
    float time = 0f;
    public float speed;

    public PlayerShip ship;

    // Use this for initialization
    void Start()
    {
        _frequency = frequency;
        particles = GetComponent<ParticleSystem>();
        CreatePoints();
    }

    // Update is called once per frame
    void Update()
    {
        if (frequency != _frequency)
        {
            CalcNewFreq();
            CreatePoints();
        }

        float yPos = Mathf.Sin(Time.time * _frequency + phase) * amplitude;
        float xPos = Time.time * speed;

        transform.position = new Vector2(xPos, yPos);

        //if (currentResolution != resolution || points == null)
        //{
        //    CreatePoints();
        //}

        for (int i = 0; i < resolution; i++)
        {
            Vector3 p = points[i].position;
            p.y = Mathf.Sin(p.x * _frequency + phase) * amplitude;
            points[i].position = p;
            //Color c = points[i].color;
            //c.g = p.y;
            //points[i].color = c;
        }
        particles.SetParticles(points, points.Length);
    }

    private void CreatePoints()
    {
        currentResolution = resolution;
        points = new ParticleSystem.Particle[resolution];
        float increment = 1f / (resolution - 1);
        float totalAmount = resolution * length;
        for (int i = 0; i < totalAmount; i++)
        {
            float x = i * increment;
            points[i].position = new Vector3(x, 0f, 0f);
            //points[i].color = new Color(x, 0f, 0f);
            //points[i].size = 0.1f;
        }
    }

    void CalcNewFreq()
    {
        float curr = (Time.time * _frequency + phase) % (2.0f * Mathf.PI);
        float next = (Time.time * frequency) % (2.0f * Mathf.PI);
        phase = curr - next;
        _frequency = frequency;
    }

    private static float Linear(float x)
    {
        return x;
    }

    private static float Exponential(float x)
    {
        return x * x;
    }

    private static float Parabola(float x)
    {
        x = 2f * x - 1f;
        return x * x;
    }

    private static float Sine(float x)
    {
        return 0.5f + 0.5f * Mathf.Sin(2 * Mathf.PI * x + Time.timeSinceLevelLoad);
    }
}
